#!/usr/local/bin/python 
# $What$

'''Inserting optional tests with 'sqlgroup'
  
    It is sometimes useful to make inputs to an SQL statement
    optinal.  Doing so can be difficult, because not only must the
    test be inserted conditionally, but SQL boolean operators may or
    may not need to be inserted depending on whether other, possibly
    optional, comparisons have been done.  The 'sqlgroup' tag
    automates the conditional insertion of boolean operators.  

    The 'sqlgroup' tag is a block tag that has no attributes. It can
    have any number of 'and' and 'or' continuation tags.

    Suppose we want to find all people with a given first or nick name
    and optionally constrain the search by city and minimum and
    maximum age.  Suppose we want all inputs to be optional.  We can
    use DTML source like the following::

      <!--#sqlgroup-->
        <!--#sqlgroup-->
          <!--#sqltest name column=nick_name type=nb multiple optional-->
        <!--#or-->
          <!--#sqltest name column=first_name type=nb multiple optional-->
	<!--#/sqlgroup-->
      <!--#and-->
	<!--#sqltest home_town type=nb optional-->
      <!--#and-->
        <!--#if minimum_age-->
	   age >= <!--#sqlvar minimum_age type=int-->
	<!--#/if-->
      <!--#and-->
        <!--#if maximum_age-->
	   age <= <!--#sqlvar maximum_age type=int-->
	<!--#/if-->
      <!--#/sqlgroup-->

    This example illustrates how groups can be nested to control
    boolean evaluation order.  It also illustrates that the grouping
    facility can also be used with other DTML tags like 'if' tags.

    The 'sqlgroup' tag checks to see if text to be inserted contains
    other than whitespace characters.  If it does, then it is inserted
    with the appropriate boolean operator, as indicated by use of an
    'and' or 'or' tag, otherwise, no text is inserted.

'''
__rcs_id__='$Id: sqltest.py,v 1.7 1998/09/10 18:38:19 brian Exp $'

############################################################################
#     Copyright 
#
#       Copyright 1996 Digital Creations, L.C., 910 Princess Anne
#       Street, Suite 300, Fredericksburg, Virginia 22401 U.S.A. All
#       rights reserved.
#
############################################################################ 
__version__='$Revision: 1.7 $'[11:-2]

import sys
from DocumentTemplate.DT_Util import ParseError, parse_params, name_param
str=__builtins__['str']

from string import find, split, join, atoi, atof
from types import ListType, TupleType, StringType

class SQLTest: 
    name='sqltest'
    optional=multiple=None

    def __init__(self, args):
	args = parse_params(args, name='', type=None, column=None,
			    multiple=1, optional=1)
	self.__name__ = name_param(args,'sqlvar')
	has_key=args.has_key
	if not has_key('type'):
	    raise ParseError, ('the type attribute is required', 'dtvar')
	self.type=t=args['type']
	if not valid_type(t):
	    raise ParseError, ('invalid type, %s' % t, 'dtvar')
	if has_key('optional'): self.optional=args['optional']
	if has_key('multiple'): self.multiple=args['multiple']
	if has_key('column'): self.column=args['column']
	else: self.column=self.__name__

    def render(self, md):
	name=self.__name__
	t=self.type
	try: v = md[name]
	except KeyError, key:
	    if key==name and self.optional: return ''
	    raise KeyError, key, sys.exc_traceback
	    
	
	if type(v) in (ListType, TupleType):
	    if len(v) > 1 and not self.multiple:
		raise 'Multiple Values', (
		    'multiple values are not allowed for <em>%s</em>'
		    % name)
	else: v=[v]
	
	vs=[]
	for v in v:
	    if not v and type(v) is StringType and t != 'string': continue
	    if t=='int':
		try:
		    if type(v) is StringType: atoi(v)
		    else: v=str(int(v))
		except:
		    raise ValueError, (
			'Invalid integer value for <em>%s</em>' % name)
	    elif t=='float':
		if not v and type(v) is StringType: continue
		try:
		    if type(v) is StringType: atof(v)
		    else: v=str(float(v))
		except:
		    raise ValueError, (
			'Invalid floating-point value for <em>%s</em>' % name)
	    else:
		v=str(v)
		v=md.getitem('sql_quote__',0)(v)
	        #if find(v,"\'") >= 0: v=join(split(v,"\'"),"''")
		#v="'%s'" % v
    
	    vs.append(v)

	if not vs:
	    if self.optional: return ''
	    raise 'Missing Input', (
		'No input was provided for <em>%s</em>' % name)

	if len(vs) > 1:
	    vs=join(map(str,vs),', ')
	    return "%s in (%s)" % (self.column,vs)
	return "%s=%s" % (self.column,vs[0])

    __call__=render

valid_type={'int':1, 'float':1, 'string':1, 'nb': 1}.has_key

############################################################################
# $Log: sqltest.py,v $
# Revision 1.7  1998/09/10 18:38:19  brian
# Fixed missing sys import in sqltest
#
# Revision 1.6  1998/09/04 20:45:02  jim
# fixed namespace screw up due to from DT_Util import *
#
# Revision 1.5  1998/04/27 18:58:39  jim
# Now use exported sql_quote__ function to quote strings.
#
# Revision 1.4  1998/04/27 18:57:23  jim
# Now use exported sql quote function to quote strings.
#
# Revision 1.3  1998/04/15 14:56:13  jim
# Fixed problem in handling optional tests.
#
# Revision 1.2  1998/03/18 13:44:51  jim
# Fixed bug in rendering SQL in tests.
#
# Revision 1.1  1998/03/17 19:31:22  jim
# added new sql tags
#
#
